import matplotlib.pyplot as plt
import pandas as pd
from fpdf import FPDF
from datetime import date
import time

def output_pdf(file_name,vtrajectory, acceleration, velocity, drift_dist, timer, stability_cal, Xe, Ye, Ze, motor_time, motor_thrust, rocket):
    pdf = FPDF()
    pdf.add_page()
    pdf.set_font('Arial', '', 16)
    t = time.localtime()
    current_time = time.strftime("%H:%M:%S", t)
    ch = 8
    class PDF(FPDF):
        def __init__(self):
            super().__init__()
        def header(self):
            self.set_font('Arial', '', 12)
            self.cell(0, 8, 'Flight Simulation Report by Helios', 0, 1, 'C')
        def footer(self):
            self.set_y(-15)
            self.set_font('Arial', '', 12)
            self.cell(0, 8, f'Page {self.page_no()}', 0, 0, 'C')
    
    launch_details = pd.DataFrame(
          {'features' : ['Initial Yaw', 
                          'Initial Pitch', 
                          'Mean Wind Speed - North', 
                          'Mean Wind Speed - East',
                          'Turbulance Intensity',
                          'Launch Rail Length'],
           'values' : [rocket.yaw, 
                          rocket.pitch, 
                          rocket.wind_north, 
                          rocket.wind_east,
                          rocket.turb_inten,
                          rocket.rail_length]
          })
    
    Rocket_details = pd.DataFrame(
          {'features' : ['Rocket Mass', 
                          'Rocket CG', 
                          'Rocket Axial Moment of Inertia', 
                          'Rocket Transverse Moment of Inertia',
                          'Rocket Diameter',
                          'Rocket Radius',
                          'Rocket Length'],
           'values' : [rocket.rocket_mass, 
                          rocket.rocket_cg, 
                          rocket.rocket_ami, 
                          rocket.rocket_tmi,
                          rocket.rocket_dia,
                          rocket.rocket_rad,
                          rocket.rocket_length]
          })
    
    Fin_details = pd.DataFrame(
          {'features' : ['Fin Root Chord', 
                          'Fin Tip Chord', 
                          'Fin height', 
                          'Fin Sweep Length',
                          'Fin Distance from Nosetip'],
           'values' : [rocket.fin_root_chord, 
                          rocket.fin_tip_chord, 
                          rocket.fin_height, 
                          rocket.fin_sweep_length,
                          rocket.fin_dist_nosetip,]
          })
    Parachute_details = pd.DataFrame(
          {'features' : ['Parachute Diameter', 
                          'Reefed Parachute Diameter', 
                          'Reefed Height'],
           'values' : [rocket.parachute_dia, 
                          rocket.reefed_parachute_dia, 
                          rocket.reefed_height]
          })
    
    Motor_details = pd.DataFrame(
          {'features' : ['Motor Length', 
                          'Motor Diameter', 
                          'Motor Wet Mass', 
                          'Motor Dry Mass',
                          'Nozzle Mass',
                          'Nozzle Length'],
           'values' : [rocket.motor_length, 
                          rocket.motor_odia, 
                          rocket.motor_wetmass,
                          rocket.motor_drymass,
                          rocket.noz_mass,
                          rocket.noz_length]
          })
    
    pdf = PDF()
    pdf.add_page()
    pdf.set_font('Arial', 'B', 24)
    pdf.cell(w=0, h=20, txt="Helios Simulation Report", ln=1)
    pdf.set_font('Arial', '', 16)
    pdf.cell(w=30, h=ch, txt=str(date.today()), ln=1)
    pdf.cell(w=30, h=ch, txt=str(current_time), ln=2)
    pdf.ln(ch)
    pdf.multi_cell(w=0, h=5, txt="User required ")
    pdf.ln(ch)
    pdf.multi_cell(w=0, h=5, txt="Launch Details")

    for i in range(0, len(launch_details)):
        pdf.cell(w=80, h=8, 
                txt=launch_details['features'].iloc[i], 
                border=1, ln=0, align='C')
        pdf.cell(w=80, h=8, 
                txt=launch_details['values'].iloc[i].astype(str), 
                border=1, ln=1, align='C')
    pdf.ln(ch)

    pdf.multi_cell(w=0, h=5, txt="Fin Details")
    for i in range(0, len(Fin_details)):
        pdf.cell(w=80, h=8, 
                txt=Fin_details['features'].iloc[i], 
                border=1, ln=0, align='C')
        pdf.cell(w=80, h=8, 
                txt=Fin_details['values'].iloc[i].astype(str), 
                border=1, ln=1, align='C')
    pdf.ln(ch)

    pdf.multi_cell(w=0, h=5, txt="Parachute Details")
    for i in range(0, len(Parachute_details)):
        pdf.cell(w=80, h=8, 
                txt=Parachute_details['features'].iloc[i], 
                border=1, ln=0, align='C')
        pdf.cell(w=80, h=8, 
                txt=Parachute_details['values'].iloc[i].astype(str), 
                border=1, ln=1, align='C')
    pdf.ln(ch)
        
    pdf.multi_cell(w=0, h=5, txt="Motor Details")
    for i in range(0, len(Motor_details)):
        pdf.cell(w=80, h=8, 
                txt=Motor_details['features'].iloc[i], 
                border=1, ln=0, align='C')
        pdf.cell(w=80, h=8, 
                txt=Motor_details['values'].iloc[i].astype(str), 
                border=1, ln=1, align='C')
    pdf.ln(ch)
    pdf.multi_cell(w=0, h=5, txt="Results of the Simulation")
    
    plt.plot(motor_time, motor_thrust)
    plt.xlabel('Time')
    plt.ylabel('Thrust')
    plt.title('Thrust-Time Curve')
    plt.savefig('./plots/thrust_time.png', 
           transparent=False,  
           facecolor='white', 
           bbox_inches="tight")
    
    plt.plot(timer, stability_cal)
    plt.xlabel('Time')
    plt.ylabel('Stability Caliber')
    plt.title('Stability Caliber-Time Curve')
    plt.savefig('./plots/stabcal_time.png', 
           transparent=False,  
           facecolor='white', 
           bbox_inches="tight")
    
    plt.plot(timer, velocity)
    plt.xlabel('Time')
    plt.ylabel('Velocity')
    plt.title('Velocity-Time Curve')
    plt.savefig('./plots/vel_time.png', 
           transparent=False,  
           facecolor='white', 
           bbox_inches="tight")

    plt.plot(timer, acceleration)
    plt.xlabel('Time')
    plt.ylabel('Acceleration')
    plt.title('Acceleration-Time Curve')
    plt.savefig('./plots/accel_time.png', 
           transparent=False,  
           facecolor='white', 
           bbox_inches="tight")
    
    plt.plot(timer, Xe)
    plt.xlabel('Time')
    plt.ylabel('Altitude')
    plt.title('Altitude-Time Curve')
    plt.savefig('./plots/alt_time.png', 
           transparent=False,  
           facecolor='white', 
           bbox_inches="tight")

    pdf.image('./plots/thrust_time.png', 
          x = 10, y = None, w = 100, h = 0, type = 'PNG')
    
    pdf.image('./plots/stabcal_time.png', 
      x = 10, y = None, w = 100, h = 0, type = 'PNG')

    pdf.image('./plots/vel_time.png', 
      x = 10, y = None, w = 100, h = 0, type = 'PNG')

    pdf.image('./plots/accel_time.png', 
      x = 10, y = None, w = 100, h = 0, type = 'PNG')

    pdf.image('./plots/alt_time.png', 
      x = 10, y = None, w = 100, h = 0, type = 'PNG')

    pdf.output(file_name)


"""
File: ./import_data.py

Description: This function imports the aerodynamic, propulsion and rocket data which is then used to compute the flight trajectory within the driver function.
The data here is passed in the form of .xlsx files, examples for which have been provided in the repository.
    - rocket_data.xlsx
    - motor_data.xlsx
    - aero_data.xlsx
    These .xlsx files contain important information regarding the rocket, its aerodynamics and propulsion.

Function inputs:
                - rocket_data = rocket_data.xlsx (default)
                - motor_data = motor_data.xlsx (default)
                - aero_data = aero_data.xlsx (default)

Function outputs:
                - rmas --> dry mass of the rocket without the motor
                - rcg --> rocket center of gravity without the motor
                - AMI --> axial moment of inertia of the rocket without the motor
                - TMI --> transverse moment of inertia of the rocket without the motor
                - mrad --> radius of the rocket motor
                - rlen --> length of the rocket
                - np.array(fin_details) --> fin details which includes the root chord, tip chord, height, sweep length and distance from nosetip 
                - Aref --> reference area of the rocket
                - mpos --> length of the rocket
                - data1 --> aerodynamic data of the rocket without airbrakes
                - data2 --> aerodynamic data of the rocket with airbrakes
                - motor_data --> propulsion data
                - length --> length of the motor 
                - odia --> outer diameter of the rocket
                - wetmass --> mass of the motor
                - drymass --> dry mass of the rocket
                - nozmass --> mass of the nozzle
                - nozlen --> length of the nozzle
                - thrust --> motor thrust data
                - time --> motor burn time data
"""


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

prop_data = pd.read_excel('prop_data.xlsx')
thrust = prop_data.loc[:,'thrust'].to_list()
time = prop_data.loc[:,'time'].to_list()

fig, ax = plt.subplots(5,2)
ax[0][0].plot(time, thrust)
ax[0][0].set_title("Thrust-Time Curve")
ax[0][0].set_xlabel("Time")
ax[0][0].set_ylabel("Thrust")
ax[0][1].plot(time, thrust)
ax[1][0].plot(time, thrust)
ax[1][1].plot(time, thrust)
ax[2][0].plot(time, thrust)
ax[2][1].plot(time, thrust)
ax[3][0].plot(time, thrust)
ax[3][1].plot(time, thrust)
ax[4][0].plot(time, thrust)
ax[4][1].plot(time, thrust)


plt.show()


"""
File: ./rocket_sim.py

Description: This is the driver function for the simulator
"""
import math
import pandas as pd
import numpy as np
from .initialize import rocket_params
from .density_and_temp import density_and_temp
from .propulsion_para import propulsion_para
from .aerodynamic_para import aerodynamic_para
from .predictor import predictor
from .turbulance_generator import turbulence_generator
from .wind_forces import wind_forces
from .Rk4Derivative import Rk4Derivative
from .Rk4Utils import Rk4Utils
rocketSim = rocket_params()


def rocket_sim(rocket):
    counter = 1
    prop_data = pd.read_excel(rocket.prop_data)
    thrust = prop_data.loc[:,'thrust'].to_list()
    time = prop_data.loc[:,'time'].to_list()
    aero_data_wab = pd.read_excel(rocket.aero_data, sheet_name=0)
    aero_data_ab = pd.read_excel(rocket.aero_data, sheet_name=1)
    Aref = math.pi*(rocket.rocket_dia**2)/4
    fin_details = [rocket.fin_root_chord,rocket.fin_tip_chord,rocket.fin_height,rocket.fin_sweep_length,rocket.fin_dist_nosetip]
    while rocketSim.Xe[counter] >= 0 or rocketSim.timer[counter]<time[-1]:
        sphi = math.sin(rocketSim.phi[counter])
        stheta = math.sin(rocketSim.theta[counter])
        spsi = math.sin(rocketSim.psi[counter])
        cphi = math.cos(rocketSim.phi[counter])
        ctheta = math.cos(rocketSim.theta[counter])
        cpsi = math.cos(rocketSim.psi[counter])

        mydensity, mytemperature = density_and_temp(rocketSim.Xe[counter], rocketSim.temperature)
        v_sound = math.sqrt(1.4*287*mytemperature)
        mach_no = rocketSim.u[counter]/v_sound

        mmass, mcg, mIx, mIy, mythrust = propulsion_para(rocketSim.timer[counter], 
                                                         rocket.motor_length, 
                                                         rocket.motor_odia, 
                                                         rocket.motor_wetmass,
                                                         rocket.motor_drymass,
                                                         rocket.noz_mass,
                                                         rocket.noz_length,
                                                         time, 
                                                         thrust
                                                         )
        rocketSim.Mass[counter] = rocket.rocket_mass + mmass
        rocketSim.CG[counter] = ((rocket.rocket_mass*rocket.rocket_cg)+(mmass*(rocket.rocket_length-(rocket.motor_length+rocket.noz_length)+mcg)))/rocketSim.Mass[counter]
        rocketSim.Ixx[counter] = rocket.rocket_ami + mIx
        rocketSim.Iyy[counter] = rocket.rocket_tmi + (rocket.rocket_mass*(rocketSim.CG[counter]-(rocket.rocket_cg))**2)+(mIy)+(mmass*(rocketSim.CG[counter]-(rocket.rocket_length-rocket.motor_length+mcg))**2)

        if rocketSim.timer[counter] > time[-1]:
            rocketSim.mphase = 0
        if rocketSim.phase == 0:
            rocketSim.CP[counter], rocketSim.Cn_pitch[counter], rocketSim.Cn_yaw[counter], rocketSim.Cn_alpha[counter], rocketSim.Cd[counter] = aerodynamic_para(aero_data_wab,
                                                                                                                                                                 aero_data_ab,
                                                                                                                                                                 mach_no,
                                                                                                                                                                 rocketSim.psi[counter],
                                                                                                                                                                 rocketSim.theta[counter],
                                                                                                                                                                 rocketSim.mphase,
                                                                                                                                                                 rocketSim.flag)
        momentarm = rocketSim.CP[counter]-rocketSim.CG[counter] 
        rocketSim.Stab_Cal[counter] = momentarm/(rocket.rocket_dia)

        if rocketSim.timer[counter] > time[-1]:
            if rocketSim.flag == 0:
                rocketSim.prediction[counter] = predictor(aero_data_wab, 
                                                          rocketSim.acceleration[counter], 
                                                          rocketSim.u[counter],
                                                          rocketSim.Xe[counter],
                                                          rocketSim.Mass[counter],
                                                          Aref,
                                                          mytemperature)
                
            elif rocketSim.flag == 1:
                rocketSim.prediction[counter] = predictor(aero_data_ab, 
                                                          rocketSim.acceleration[counter], 
                                                          rocketSim.u[counter],
                                                          rocketSim.Xe[counter],
                                                          rocketSim.Mass[counter],
                                                          Aref,
                                                          mytemperature)
            
            if rocketSim.prediction[counter] > rocket.desired_apogee:
                if rocketSim.timer[counter] - rocket.delaytracker >= rocket.delay:
                    rocketSim.flag = 1
                    rocket.delaytracker = rocketSim.timer[counter]
            
            elif rocketSim.prediction[counter] < rocket.desired_apogee:
                if rocketSim.timer[counter] - rocket.delaytracker >= rocket.delay:
                    rocketSim.flag = 0
                    rocket.delaytracker = rocketSim.timer[counter]
        
        else:
            rocketSim.prediction[counter] = 0
        
        if rocketSim.flag == 1:
            rocketSim.state[counter] = 2000
        
        else:
            rocketSim.state[counter] = 500
        
        if rocketSim.timer[counter] < 10 or rocketSim.u[counter] >= 0:
            sidey, rocketSim.turb_gen = turbulence_generator(counter,rocketSim.turb_gen,rocket.wind_north,rocket.turb_inten,rocket.time_step)
            sidez, rocketSim.turb_gen = turbulence_generator(counter,rocketSim.turb_gen,rocket.wind_east,rocket.turb_inten,rocket.time_step)
        
            Fy, Mz = wind_forces(sidey,rocketSim.CG[counter],mydensity,rocket.rocket_rad,rocket.rocket_length,fin_details)
            Fz, My = wind_forces(sidez,rocketSim.CG[counter],mydensity,rocket.rocket_rad,rocket.rocket_length,fin_details)

            Fy = Fy*np.sign(sidey)*cpsi
            Fz = Fz*np.sign(sidez)*ctheta
            My = My*np.sign(sidez)*ctheta
            Mz = -Mz*np.sign(sidey)*cpsi
            
            if rocketSim.Xe[counter] > rocket.rail_length:
                CNY = -(rocketSim.Cn_yaw[counter]*np.sign(rocketSim.psi[counter]))-(rocketSim.Cd[counter]*spsi)-(rocketSim.Cn_alpha[counter]*math.atan((rocketSim.q[counter]*momentarm)/rocketSim.u[counter]))
                CNP= -(rocketSim.Cn_pitch[counter]*np.sign(rocketSim.theta[counter]))-(rocketSim.Cd[counter]*stheta)-(rocketSim.Cn_alpha[counter]*math.atan((rocketSim.p[counter]*momentarm)/rocketSim.u[counter]))
                rocketSim.C_roll[counter]= -2*math.pi*math.atan((1.5*rocket.rocket_rad*rocketSim.r[counter])/rocketSim.u[counter])*np.sign(rocketSim.r[counter])
                
            else:
                CNY = 0
                CNP = 0
                rocketSim.C_roll[counter] = 0
                My = 0
                Mz = 0
                Fy = 0
                Fz = 0
            
            drag = 0.5*rocketSim.Cd[counter]*rocketSim.u[counter]*rocketSim.u[counter]*mydensity*Aref
            Yaw = (0.5*CNY*mydensity*rocketSim.u[counter]*rocketSim.u[counter]*Aref*momentarm) + Mz
            Pitch = (0.5*CNP*mydensity*rocketSim.u[counter]*rocketSim.u[counter]*Aref*momentarm) + My
            Roll = rocketSim.C_roll[counter]*mydensity*rocketSim.u[counter]*rocketSim.u[counter]*(fin_details[2]*(fin_details[0]+fin_details[1]))*(1.5*rocket.rocket_rad)
            
            rocketSim.vtrajectory[counter,0]=rocketSim.Xe[counter]
            rocketSim.vtrajectory[counter,1]=rocketSim.Ye[counter]
            rocketSim.vtrajectory[counter,2]=rocketSim.Ze[counter]
        
        else:
            pdia = rocket.reefed_parachute_dia
            if rocketSim.Xe[counter]<rocket.reefed_height:
                pdia = rocket.parachute_dia
                drag = (0.5*1.2*math.pi*(pdia)*(pdia)/4)*mydensity*rocketSim.u[counter]*rocketSim.u[counter]*np.sign(rocketSim.u[counter])
                Fy = (0.5*0.1*mydensity*(sidey**2)*(math.pi*(pdia)*(pdia)/4)*np.sign(sidey))-(0.5*0.1*mydensity*(rocketSim.v[counter]**2)*(math.pi*(pdia)*(pdia)/4)*np.sign(rocketSim.v[counter]))
                Fz = (0.5*0.1*mydensity*(sidez**2)*(math.pi*(pdia)*(pdia)/4)*np.sign(sidez))-(0.5*0.1*mydensity*(rocketSim.w[counter]**2)*(math.pi*(pdia)*(pdia)/4)*np.sign(rocketSim.w[counter]))
                Yaw = 0
                Pitch = 0
                Roll = 0
                rocketSim.phi[counter] = 0
                rocketSim.psi[counter] = 0
                rocketSim.theta[counter] = 0
                rocketSim.phase = 1
                rocketSim.flag = 0

        Rk1 = Rk4Utils(Rk4Derivative(mythrust,
                                     drag,
                                     Fy,
                                     Fz,
                                     rocketSim.Mass[counter],
                                     rocketSim.u[counter],
                                     rocketSim.v[counter],
                                     rocketSim.w[counter],
                                     rocketSim.p[counter],
                                     rocketSim.q[counter],
                                     rocketSim.r[counter],
                                     cphi,cpsi,ctheta,sphi,spsi,stheta,
                                     Yaw,Pitch,Roll,
                                     rocketSim.Ixx[counter],
                                     rocketSim.Iyy[counter],rocketSim.Iyy[counter]))
        Rk2 = Rk4Utils(Rk4Derivative(mythrust,
                                     drag,
                                     Fy,
                                     Fz,
                                     rocketSim.Mass[counter],
                                     rocketSim.u[counter] + Rk1.udot*(rocket.time_step/2),
                                     rocketSim.v[counter] + Rk1.vdot*(rocket.time_step/2),
                                     rocketSim.w[counter] + Rk1.wdot*(rocket.time_step/2),
                                     rocketSim.p[counter] + Rk1.pdot*(rocket.time_step/2),
                                     rocketSim.q[counter] + Rk1.qdot*(rocket.time_step/2),
                                     rocketSim.r[counter] + Rk1.rdot*(rocket.time_step/2),
                                     cphi,cpsi,ctheta,sphi,spsi,stheta,
                                     Yaw,Pitch,Roll,
                                     rocketSim.Ixx[counter],
                                     rocketSim.Iyy[counter],rocketSim.Iyy[counter]))
        
        Rk3 = Rk4Utils(Rk4Derivative(mythrust,
                                     drag,
                                     Fy,
                                     Fz,
                                     rocketSim.Mass[counter],
                                     rocketSim.u[counter] + Rk2.udot*(rocket.time_step/2),
                                     rocketSim.v[counter] + Rk2.vdot*(rocket.time_step/2),
                                     rocketSim.w[counter] + Rk2.wdot*(rocket.time_step/2),
                                     rocketSim.p[counter] + Rk2.pdot*(rocket.time_step/2),
                                     rocketSim.q[counter] + Rk2.qdot*(rocket.time_step/2),
                                     rocketSim.r[counter] + Rk2.rdot*(rocket.time_step/2),
                                     cphi,cpsi,ctheta,sphi,spsi,stheta,
                                     Yaw,Pitch,Roll,
                                     rocketSim.Ixx[counter],
                                     rocketSim.Iyy[counter],rocketSim.Iyy[counter]))
        
        Rk4 = Rk4Utils(Rk4Derivative(mythrust,
                                     drag,
                                     Fy,
                                     Fz,
                                     rocketSim.Mass[counter],
                                     rocketSim.u[counter] + Rk3.udot*(rocket.time_step/2),
                                     rocketSim.v[counter] + Rk3.vdot*(rocket.time_step/2),
                                     rocketSim.w[counter] + Rk3.wdot*(rocket.time_step/2),
                                     rocketSim.p[counter] + Rk3.pdot*(rocket.time_step/2),
                                     rocketSim.q[counter] + Rk3.qdot*(rocket.time_step/2),
                                     rocketSim.r[counter] + Rk3.rdot*(rocket.time_step/2),
                                     cphi,cpsi,ctheta,sphi,spsi,stheta,
                                     Yaw,Pitch,Roll,
                                     rocketSim.Ixx[counter],
                                     rocketSim.Iyy[counter],rocketSim.Iyy[counter]))
        
        rocketSim.u[counter+1] = rocketSim.u[counter] + (1/6)*(Rk1.udot + 2*Rk2.udot + 2*Rk3.udot + Rk4.udot)*rocket.time_step
        rocketSim.acceleration[counter+1] =  (1/6)*(Rk1.udot + 2*Rk2.udot + 2*Rk3.udot + Rk4.udot)
        rocketSim.v[counter+1] = rocketSim.v[counter] + (1/6)*(Rk1.vdot + 2*Rk2.vdot + 2*Rk3.vdot + Rk4.vdot)*rocket.time_step
        rocketSim.w[counter+1] = rocketSim.w[counter] + (1/6)*(Rk1.wdot + 2*Rk2.wdot + 2*Rk3.wdot + Rk4.wdot)*rocket.time_step
        rocketSim.r[counter+1] = rocketSim.r[counter] + (1/6)*(Rk1.rdot + 2*Rk2.rdot + 2*Rk3.rdot + Rk4.rdot)*rocket.time_step
        rocketSim.q[counter+1] = rocketSim.q[counter] + (1/6)*(Rk1.qdot + 2*Rk2.qdot + 2*Rk3.qdot + Rk4.qdot)*rocket.time_step
        rocketSim.p[counter+1] = rocketSim.p[counter] + (1/6)*(Rk1.pdot + 2*Rk2.pdot + 2*Rk3.pdot + Rk4.pdot)*rocket.time_step
        rocketSim.phi[counter+1] = rocketSim.phi[counter] + (1/6)*(Rk1.phidot + 2*Rk2.phidot + 2*Rk3.phidot + Rk4.phidot)*rocket.time_step
        rocketSim.theta[counter+1] = rocketSim.theta[counter] + (1/6)*(Rk1.thetadot + 2*Rk2.thetadot + 2*Rk3.thetadot + Rk4.thetadot)*rocket.time_step
        rocketSim.psi[counter+1] = rocketSim.psi[counter] + (1/6)*(Rk1.psidot + 2*Rk2.psidot + 2*Rk3.psidot + Rk4.psidot)*rocket.time_step
        rocketSim.Xe[counter+1] = rocketSim.Xe[counter] + (1/6)*(Rk1.xedot + 2*Rk2.xedot + 2*Rk3.xedot + Rk4.xedot)*rocket.time_step
        rocketSim.Ye[counter+1] = rocketSim.Ye[counter] + (1/6)*(Rk1.yedot + 2*Rk2.yedot + 2*Rk3.yedot + Rk4.yedot)*rocket.time_step
        rocketSim.Ze[counter+1] = rocketSim.Ze[counter] + (1/6)*(Rk1.zedot + 2*Rk2.zedot + 2*Rk3.zedot + Rk4.zedot)*rocket.time_step
        rocketSim.velx[counter+1] = (1/6)*(Rk1.xedot + 2*Rk2.xedot + 2*Rk3.xedot + Rk4.xedot)
        rocketSim.timer[counter+1] = rocketSim.timer[counter] + rocket.time_step
        counter = counter + 1
        rocketSim.drift[counter] = np.sqrt((rocketSim.Ye[counter]**2)+(rocketSim.Ze[counter]**2))

        if rocketSim.phase == 0 and rocketSim.Xe[counter]<0:
            rocketSim.Xe[counter] = 0
            rocketSim.u[counter] = 0

    rocketSim.prediction[counter] = 0
    rocketSim.state[counter] = 500
    return rocketSim.vtrajectory, rocketSim.acceleration, rocketSim.velx, rocketSim.drift, rocketSim.timer, rocketSim.Stab_Cal, rocketSim.Xe, rocketSim.Ye,rocketSim.Ze, time, thrust


"""
File: ./wind_forces.py

Description:
        The function calculates the drag forces and moments exerted on the rocket due to the wind gusts that are randomly generated
        due to turbulence. The moment on the the rocket body is neglected as it is axisymmetric and only the moment on the fins
        is accounted for.
        
Function Input:
        vel --> Velocity of the wind
        cm --> Center of gravity
        den --> Density if the air at a given time interval
        rad --> Radius of the rocket
        len --> Length of the rocket
        fin --> A [5x1] Vector that holds the details of the fins in the form [Root Chord, Tip Chrod, Height, Sweep Length, Distance from Nosetip]

Function Output:
        F --> Drag force experienced by the fins due to wind gusts
        mom --> Moment due to Drag force experienced by the fins due to wind gusts
"""

import numpy as np
from scipy.interpolate import interp1d

def wind_forces(vel,cm,den,rad,len,fin):
    vel = np.sqrt(vel*vel)   
    cdcyl=[[-5,5],[10 ,3.33],[100 ,1.334],[1000 ,0.738],[10000 ,0.766],[100000 ,0.56],[1000000 ,0.55],[10000000, 0.54],[100000000 ,0.54]]
    cdcyl_1 = []
    cdcyl_2 = []
    for i in cdcyl:
        cdcyl_1.append(i[0])
        cdcyl_2.append(i[1])
    cdplt=1.28;  
    ryno=(1.225*vel**2*rad)/(0.0000173);    
    f = interp1d(cdcyl_1,cdcyl_2, kind = 'linear')
    cn = f(ryno)
    F = 0.5* den* vel**2*((cn*rad*2*len)*(cdplt*(fin[0]+fin[1])*fin[2]))
    mom = 0.5*den*vel**2*cdplt*(fin[0]+fin[1])*fin[2]*(fin[4]-cm)
    return F,mom


"""
File: ./propulsion_para.py

Description:
        This function takes in multiple motor parameters like it's Wet and Dry mass, Diameter, Nozzle's mass and length, Thrust vs Time to compute 
        Motor's mass, position of center of gravity, and the Moment of Inertia using kinematics equations. The fucntion also interpolates
        the thrust generated for the given motor at every time step using the Thrust vs Time data.

Function Inputs:
        timer --> Current timestep
        Length --> Length of the Motor
        odia --> Outer Diameter of the motor
        wetmass --> Wetmass of the motor
        drymass --> Drymass of the motor
        nozmass --> Nozzle mass of the motor
        nozlen --> Length of the Nozzle
        time --> Time data for the given motor
        thrust --> Thrust data for the given motor

Function Outputs:
        mass --> Mass of the entire motor (Drymass + Wetmass)
        cg --> Position of Center of gravity of the motor
        Ix --> Moment of Inertia along X
        Iy --> Moment of Inertia along Y
        currentthrust --> Thrust for the given timestep
"""
from scipy.interpolate import interp1d
def propulsion_para(timer, length, odia, wetmass, drymass, nozmass, nozlen, time, thrust):
    if timer>time[-1]:
        currentthrust = 0
        mass = drymass+nozmass #total mass
        Ix1 = 0 #MI of motor grain
        Ix2 = drymass*(odia/2)**2 #MI of casing assuming it is a thin cylinder
        Ix3 = 0.5*nozmass*0.25*(odia**2)
        cg = (((mass-nozmass)*(length/2))+(nozmass*(length+(nozlen/2))))/mass
        Iy1 = 0
        Iy2 = (0.5*drymass*(odia/2)**2)+((1/12)*drymass*length**2)+(drymass*(cg-(length/2))**2)
        Iy3 = ((1/12)*nozmass*(((3/4)*(odia**2))+nozlen**2))+(nozmass*(cg-(length+(nozlen/2)))**2)
    else:
        f = interp1d(time, thrust, kind='cubic', fill_value="extrapolate")
        currentthrust = f(timer) 
        burnrate = (wetmass-drymass)/(time[-1])
        dm = burnrate*timer
        mass = (wetmass-dm)+nozmass
        Ix1 = 0.5*(mass-drymass-nozmass)*0.25*(odia**2)
        Ix2= drymass*(odia/2)**2
        Ix3= 0.5*nozmass*0.25*(odia**2)
        cg= (((drymass)*(length/2))+((mass-drymass)*((length)/2))+(nozmass*(length+(nozlen/2))))/mass
        Iy1= ((1/12)*(mass-drymass)*(((3/4)*(odia**2))+(length)**2))+((mass-drymass)*(cg-((length)/2))**2)
        Iy2= (0.5*drymass*(odia/2)**2)+((1/12)*drymass*length**2)+(drymass*(cg-(length/2))**2)
        Iy3=((1/12)*nozmass*(((3/4)*(odia**2))+nozlen**2))+(nozmass*(cg-(length+(nozlen/2)))**2)
    Ix = Ix1+Ix2+Ix3
    Iy = Iy1+Iy2+Iy3
    return mass,cg,Ix,Iy,currentthrust


import argparse
import os
from dotenv import load_dotenv
from .rocket_sim import rocket_sim
from .output_pdf import output_pdf
load_dotenv()

def main():
    p = argparse.ArgumentParser()

    p.add_argument("--yaw", help="Initial Yaw in degrees", default=os.getenv('INIT_YAW'), type=float)
    p.add_argument("--pitch", help="Initial Pitch in degrees", default=os.getenv('INIT_PITCH'), type=float)
    p.add_argument("--wind-north", help="Mean wind speed in North", default=os.getenv('WIND_VEL_N'), type=float)
    p.add_argument("--wind-east", help="Mean wind speed in East", default=os.getenv('WIND_VEL_E'), type=float)
    p.add_argument("--turb-inten", help="Turbulance Intensity Percentage", default=os.getenv('TURB_INTEN'), type=float)
    p.add_argument("--rail-length", help="Mean wind speed in North", default=os.getenv('LAUNCH_RAIL_LEN'), type=float)
    p.add_argument("--air-brakes", help="Air Brakes", default=os.getenv('AIRBRAKES'))
    p.add_argument("--desired-apogee", help="Desired Apogee", default=os.getenv('DESIRED'), type=float)
    p.add_argument("--parachute-dia", help="Dis-reefed Parachute Diameter", default=os.getenv('PARACHUTE_DIA'), type=float)
    p.add_argument("--reefed-parachute-dia", help="Reefed Parachute Diameter", default=os.getenv('REEFED_DIA'), type=float)
    p.add_argument("--reefed-height", help="Height at which the parachute is dis-reefed", default=os.getenv('REEFED_HEIGHT'), type=float)
    p.add_argument("--rocket-mass", help="Mass of the Rocket", default=os.getenv('ROCKET_MASS'), type=float)
    p.add_argument("--rocket-cg", help="Center of Gravity of Rocket", default=os.getenv('ROCKET_CG'), type=float)
    p.add_argument("--rocket-ami", help="AMI of Rocket", default=os.getenv('ROCKET_AMI'), type=float)
    p.add_argument("--rocket-tmi", help="TMI of Rocket", default=os.getenv('ROCKET_TMI'), type=float)
    p.add_argument("--rocket-dia", help="Diameter of the Rocket", default=os.getenv('ROCKET_DIA'), type=float)
    p.add_argument("--rocket-rad", help="Radius of the Rocket", default=os.getenv('ROCKET_RAD'), type=float)
    p.add_argument("--rocket-length", help="Length of the Rocket", default=os.getenv('ROCKET_LENGTH'), type=float)
    p.add_argument("--fin-root-chord", help="Root Chord of Fin", default=os.getenv('FIN_ROOT_CHORD'), type=float)
    p.add_argument("--fin-tip-chord", help="Tip Chord of Fin", default=os.getenv('FIN_TIP_CHORD'), type=float)
    p.add_argument("--fin-height", help="Height of Fin", default=os.getenv('FIN_HEIGHT'), type=float)
    p.add_argument("--fin-sweep-length", help="Fin Sweep Length", default=os.getenv('FIN_SWEEP_LEN'), type=float)
    p.add_argument("--fin-dist-nosetip", help="Distance from nosetip to leading edge of fin", default=os.getenv('FIN_DIST_NOSETIP'), type=float)
    p.add_argument("--motor-length", help="Length of the motor", default=os.getenv('MOTOR_LENGTH'), type=float)
    p.add_argument("--motor-odia", help="Outer Diameter of the motor", default=os.getenv('MOTOR_ODIA'), type=float)
    p.add_argument("--motor-wetmass", help="Wetmass of the motor", default=os.getenv('MOTOR_WETMASS'), type=float)
    p.add_argument("--motor-drymass", help="Drymass of the motor", default=os.getenv('MOTOR_DRYMASS'), type=float)
    p.add_argument("--noz-mass", help="Nozzle Mass", default=os.getenv('NOZ_MASS'), type=float)
    p.add_argument("--noz-length", help="Nozzle Length", default=os.getenv('NOZ_LENGTH'), type=float)
    p.add_argument("--prop-data", help="Path to Propulsion Data", default=os.getenv('PROP_DATA'))
    p.add_argument("--aero-data", help="Path to Aerodynamics Data", default=os.getenv('AERO_DATA'))
    p.add_argument("--delaytracker", help="Keeps track of mechanical delay of airbrakes in seconds", default=os.getenv('DELAY_TRACKER'), type=float)
    p.add_argument("--delay", help="Keeps track of electrical delay of airbrakes in seconds", default=os.getenv('DELAY'), type=float)
    p.add_argument("--time-step", help="Time step for each computation", default=os.getenv('TIME_STEP'), type=float)
    p.add_argument("-o","--output", help="Name of the output file", default=os.getenv('OUTPUT'))

    args = p.parse_args()

    vtrajectory, acceleration, velocity, drift_dist, timer, stability_cal, Xe, Ye, Ze, motor_time, motor_thrust = rocket_sim(args)
    output_pdf(args.output,vtrajectory, acceleration, velocity, drift_dist, timer, stability_cal, Xe, Ye, Ze, motor_time, motor_thrust, args)




"""
File:   ./density_and_temp.py

Description:
        This function takes in the altitude (m) of the rocket at a given time interval and the temperature (K) of the outside at sea level 
        to compute density and temperature at every given time step respectively using the International Standard Atmosphere model.

Function Inputes:
        altitude --> Altitude of the rocket at a given time step in meters
        temperature_outside --> Temperature of the outside at sea level in kelvin
        a --> Lapse rate (-0.0065 for altitude below 10 km)

Function Outputs:
        density --> Density of the air at a given altitude
        temperature --> Temperature of the air at a given altitude
"""

def density_and_temp(altitude,temperature_outside):
        a = -0.0065
        temperature = temperature_outside + (a * altitude)
        density = 1.23 * ((temperature/temperature_outside)**((-9.81/(a*287))-1))
        
        return density, temperature


"""
File: ./initialize.py

Description: This function initializes all the aerodynamic and propulsion parameters 
which has to be entered by the user. This includes initialization of all force, velocity,
moment of inertia, displacement and stability related components required in a six degree 
of freedom model. 

Function Inputs: None

Function Outputs: 
                - acceleration --> acceleration of the rocket at any time interval
                - prediction --> variable to store predicted apogee at different intervals of tim-
                - u --> X-component of the velocity vector
                - v --> Y-component of the velocity vector
                - w --> Z-component of the velocity vector
                - p --> Yaw rate of the rocket in rad/s
                - q --> Pitch rate of the rocket in rad/s
                - r --> Roll rate of the rocket in rad/s
                - psi --> Yaw angle of rocket in degrees
                - theta --> Pitch angle of rocket in degrees
                - phi --> Roll angle of rocket in degrees
                - Xe --> Displacement along X-direction at any interval in meters
                - Ye --> Displacement along Y-direction at any interval in meters
                - Ze --> Displacement along Z-direction at any interval in meters
                - timer --> time at any given time step in seconds
                - velx --> Velocity along X-direction
                - counter1 --> variable to track the number of iterations
                - temp --> temperature at any given interval
                - CP --> coefficient of pressure 
                - C_roll --> coefficient for roll
                - Cn_yaw --> coefficient for yaw moment
                - Cn_pitch --> coefficient for pitch moment
                - Cn_alpha --> coefficient for angle of attack
                - Cd --> drag coefficient
                - Mass --> mass of the rocket
                - CG --> centre of gravity
                - Ixx --> moment of inertia with respect to X-axis
                - Iyy --> moment of inertia with respect to Y-axis
                - Stab_Cal --> stability caliber of the rocket
                - flag --> flag variable 
                - phase --> variable to keep track of phase of rocket (0 for ascent and 1 for parachute)
                - mphase --> determines the phase of the motors
                - state --> tracks the airbrakes state (2000 when deployed and 500 when deployed)
                - turb_gen --> generates turbulance constant
                - drift --> range calculated from the launch pad location to the landing location
                - vtrajectory --> variable for plotting rocket trajectory during ascent phase
"""
import numpy as np
class rocket_params:
    acceleration = np.zeros(300000)
    prediction = np.zeros(300000)
    u = np.zeros(300000)
    v = np.zeros(300000)
    w = np.zeros(300000)
    p = np.zeros(300000)
    q = np.zeros(300000)
    r = np.zeros(300000)
    psi = np.zeros(300000)
    theta = np.zeros(300000)
    phi = np.zeros(300000)
    Xe = np.zeros(300000)
    Ye = np.zeros(300000)
    Ze = np.zeros(300000)
    timer = np.zeros(300000)
    velx = np.zeros(300000)
    temperature = 300
    CP = np.zeros(300000)
    C_roll = np.zeros(300000)
    Cn_yaw = np.zeros(300000)
    Cn_pitch = np.zeros(300000)
    Cn_alpha = np.zeros(300000)
    Cd = np.zeros(300000)
    Mass = np.zeros(300000)
    CG = np.zeros(300000)
    Ixx = np.zeros(300000)
    Iyy = np.zeros(300000)
    Stab_Cal = np.zeros(300000)
    flag= 0
    phase = 0
    mphase = 1
    state = np.zeros(300000)
    turb_gen = np.zeros((4,4))
    turb_gen[0,0] = 1
    turb_gen[0,3] = 1.5
    drift = np.zeros(300000)
    vtrajectory = np.zeros((300000, 3))


"""
File: ./predictor.py

Description: This function is used to predict the apogee of the rocket using
rocket acceleration, velocity and displacement values. This function uses 
fourth order Runge-Kutta method to predict the apogee and velocity of the
rocket. It takes into account the drag while predicting the velocity and 
apogee values during the flight.

Function Input:
              - aerodata --> collection of data consisting of aerodynamic coefficients at different mach values
              - accelerationx --> X-component of acceleration vector
              - velocityx --> X-component of velocity vector
              - displacementx --> X-component of displacement vector
              - currentmass --> mass of the rocket at that interval
              - refArea --> reference area of the rocket
              - temp --> temperature at that time interval
              - dt --> time step
Function Output:
              - prediction --> predicted apogee of the rocket at that moment
"""
import numpy as np
from scipy.interpolate import interp1d

def predictor(aerodata,accelerationx,velocityx,displacementx,currentmass,refArea,temp):
    dt = 0.01
    accel = np.zeros(100)
    vel = np.zeros(100)
    disp = np.zeros(100)
    timer = np.zeros(100)
    counter = 1

    accel[0] = -accelerationx
    vel[0] = velocityx
    disp[0]= displacementx

    cd_data = aerodata.iloc[1:200, 2]
    mach_data = aerodata.iloc[1:200, 0]
    a = -0.0065

    while vel[counter]>0:
        mytemp = temp+(a*disp[counter])
        mydensity = 1.23*((mytemp/temp)**((-9.81/(a*287))-1))
        mach = vel[counter]/np.sqrt(1.4*287*mytemp)
        f = interp1d(mach_data, cd_data, kind='cubic')
        Cd = f(mach)
        def F(t,y,v):
            return v
        def G(t, y, v):
            return -((0.5 * Cd * mydensity * v * abs(v) * refArea) / currentmass) - 9.81
        
        k1 = F(timer[counter], disp[counter], vel[counter])
        L1 = G(timer[counter],disp[counter],vel[counter])
        k2 = F((timer[counter]+0.5*dt),(disp[counter]+0.5*dt*k1),(vel[counter]+0.5*dt*L1))
        L2 = G((timer[counter]+0.5*dt),(disp[counter]+0.5*dt*k1),(vel[counter]+0.5*dt*L1))
        k3 = F((timer[counter]+0.5*dt),(disp[counter]+0.5*dt*k2),(vel[counter]+0.5*dt*L2))
        L3 = G((timer[counter]+0.5*dt),(disp[counter]+0.5*dt*k2),(vel[counter]+0.5*dt*L2))
        k4 = F((timer[counter]+dt),(disp[counter]+k3*dt),(vel[counter]+L3*dt))
        L4 = G((timer[counter]+dt),(disp[counter]+k3*dt),(vel[counter]+L3*dt))


        disp[counter+1] = disp[counter] + dt*(1/6)*(k1 + 2*k2 + 2*k3 +  k4)
        vel[counter+1] = vel[counter] + dt*(1/6)*(L1 + 2*L2 + 2*L3 + L4)
        accel[counter+1] = G(timer[counter],disp[counter+1],vel[counter+1])
        timer[counter+1] = timer[counter] + dt
        counter = counter + 1
    
    prediction = np.max(disp)
    return prediction


"""
File:  ./turbulence_generator.py

Description:
        This function generates random gusts of wind who's magnitudes are centered around the mean of user inputed wind
        velocities which is then weighted using turbulence intensity. It also adds pink noise around the mean velocity to generate 
        turbulence.
        
Function Input:
        n --> Counter Variable
        a --> Turbuelnce generated due to gusts of wind
        U --> Mean air speed
        I --> Turbulence Intensity
        dt --> time step 

Function Output:
        u --> Velocity of the wind with noise
        a --> Turbulence generated due to gusts of wind
"""
import numpy as np

def turbulence_generator(n,a,U,I,dt):
        
        Standard_deviation = U*I*0.01
        
        if n==0:
                a[0,1] = np.random.normal()
        
        elif n==1:
                a[1,0] = (2-(5/6))*(a[0,0]/2)
                a[1,1] = np.random.normal() - (a[2,1]*a[1,2])
                a[1,2] = a[0,2] + dt
        
        elif n==2:
                a[2,0] = (3-(5/6))*(a[1,0]/3)
                a[2,1] = np.random.normal() -(a[2,0]*a[1,1])-(a[1,0]*a[0,1])
                a[2,2] = a[1,2] + dt
                avg = (a[1,1] + a[0,1])/2
                a[0,3] = np.sqrt(((a[2,1]-avg)**2 + (a[2,1]-avg)**2)/2)
        
        else:
                a[3,0] = (n-(5/6))*(a[2,0]/n)
                a[3,1] = np.random.normal() - (a[3,0]*a[2,1]) - (a[2,0]*a[1,1]) - (a[1,0]*a[0,1])
                a[3,2] = a[2,2] + dt
                avg = (a[2,1] +a[1,1] + a[0,1])/2
                a[0,3] = np.sqrt(((a[3,1]-avg)**2+(a[3,1]-avg)**2+(a[3,1]-avg)**2)/3)
        
        u = U+((a[3,1]/a[0,3])*Standard_deviation) 

        if np.sqrt((u-U)**2) > Standard_deviation:
                u = U + (np.sign(u) * Standard_deviation)
                
        return u,a


"""
File: ./aerodynamic_para.py

Description: This function file handles the aerodynamic parameters related to the model rocket. 
The parameters include Drag coefficients, Normal Force coefficients, Centre of Pressure
at different angle of attacks. This includes Normal Coefficient with respect to different
pitching and yawing at different intervals of time.  

Function Inputs:
               data1 = aero_data.xlsx sheet-1 (default)
               data2 = aero_data.xlsx sheet-2 (default)
               mach --> velocity of the Rocket at different intervals
               pitch --> pitch angle of the rocket at launch in degrees 
               yaw --> yaw angle of the rocket at launch in degrees
               phase --> defines the phase of the rocket (boost/coasting)
               flag --> airbrakes open/close counter variable
Function Outputs: 
               CP --> centre of pressure at any given interval
               CNP --> normal force coefficient with respect to pitch
               CNY --> normal force coefficient with respect to yaw
               CN_alpha --> normal force coefficient with respect to angle of attack of the rocket
               CD --> drag coefficient at any time interval
"""
import numpy as np
from scipy.interpolate import interp1d

def aerodynamic_para(data1, data2, mach, pitch, yaw, phase, flag):
    m = np.arange(0,1.99,0.01)
    cd_poweron = np.zeros((199,3,2))
    cn = np.zeros((199,3))
    cd_poweroff = np.zeros((199,3,2))
    cn_alpha = np.zeros((199,1))
    cp = np.zeros((199,1))
    angle=[0,2,4]
    mach = np.abs(mach)
    pitch = np.abs(pitch)
    yaw = np.abs(yaw)
    
#NO Airbrakes
    cd_poweron[:,0,0] = data1.iloc[0:199,4]
    cd_poweron[:,1,0] = data1.iloc[2500:2699,4]
    cd_poweron[:,2,0] = data1.iloc[5000:5199,4]
    cd_poweroff[:,0,0] = data1.iloc[0:199,3]
    cd_poweroff[:,1,0] = data1.iloc[2500:2699,3]
    cd_poweroff[:,2,0] = data1.iloc[5000:5199,3]

    cn[:,0] = data1.iloc[0:199,8]
    cn[:,1] = data1.iloc[2500:2699,8]
    cn[:,2] = data1.iloc[5000:5199,8]

    cn_alpha[:,0] = data1.iloc[0:199,11]
    cp[:,0] = data1.iloc[0:199,12]
   
#Airbrakes
    cd_poweron[:,0,1] = data2.iloc[0:199,4]
    cd_poweron[:,1,1] = data2.iloc[2500:2699,4]
    cd_poweron[:,2,1] = data2.iloc[5000:5199,4]
    cd_poweroff[:,0,1] = data2.iloc[0:199,3]
    cd_poweroff[:,1,1] = data2.iloc[2500:2699,3]
    cd_poweroff[:,2,1] = data2.iloc[5000:5199,3]

    cd_poweron = np.transpose(cd_poweron)
    cn = np.transpose(cn)
    cp = np.transpose(cp)
    cd_poweroff = np.transpose(cd_poweroff)
    cn_alpha = np.transpose(cn_alpha)
    
    if phase == 1:
        f_cd0 = interp1d(m,cd_poweron[0,0,:])
        cd0 = f_cd0(mach)
        f_cd2 = interp1d(m,cd_poweron[0,1,:])
        cd2 = f_cd2(mach)
        f_cd4 = interp1d(m,cd_poweron[0,2,:])
        cd4 = f_cd4(mach)
        coeffs_cd = np.array([cd0, cd2, cd4])
        p1 = np.polyfit(angle,coeffs_cd,2)

        CD=(p1[0] * (pitch**2+ yaw**2)) + (p1[1]*np.sqrt(pitch**2 + yaw**2)) + p1[2]

        cn0=0
        f_cn2 = interp1d(m,cn[1,:])
        cn2 = f_cd0(mach)
        f_cn4 = interp1d(m,cn[2,:])
        cn4 = f_cd2(mach)
        coeffs_cn = np.array([cn0, cn2, cn4])
        p2 = np.polyfit(angle,coeffs_cn,2)

        CNP=(p2[0]*pitch**2) +( p2[1]*pitch) + p1[2]
        CNY=(p2[0]*yaw**2) + (p2[1]*yaw) + p1[2]

        f_cp = interp1d(m,cp)
        CP = f_cp(mach)

    elif phase == 0:
        if flag == 0:
            f_cd0 = interp1d(m,cd_poweroff[0,0,:])
            cd0 = f_cd0(mach)
            f_cd2 = interp1d(m,cd_poweroff[0,1,:])
            cd2 = f_cd2(mach)
            f_cd4 = interp1d(m,cd_poweroff[0,2,:])
            cd4 = f_cd4(mach)
            coeffs_cd = np.array([cd0, cd2, cd4])
            p1 = np.polyfit(angle,coeffs_cd,2)

            CD=(p1[0] * (pitch**2+ yaw**2)) + (p1[1]*np.sqrt(pitch**2 + yaw**2)) + p1[2]

            cn0=0
            f_cn2 = interp1d(m,cn[1,:])
            cn2 = f_cd0(mach)
            f_cn4 = interp1d(m,cn[2,:])
            cn4 = f_cd2(mach)
            coeffs_cn = np.array([cn0, cn2, cn4])
            p2 = np.polyfit(angle,coeffs_cn,2)

            CNP=(p2[0]*pitch**2) +( p2[1]*pitch) + p1[2]
            CNY=(p2[0]*yaw**2) + (p2[1]*yaw) + p1[2]

            f_cp = interp1d(m,cp)
            CP = f_cp(mach)

        elif flag == 1:
            f_cd0 = interp1d(m,cd_poweroff[1,0,:])
            cd0 = f_cd0(mach)
            f_cd2 = interp1d(m,cd_poweroff[1,1,:])
            cd2 = f_cd2(mach)
            f_cd4 = interp1d(m,cd_poweroff[1,2,:])
            cd4 = f_cd4(mach)
            coeffs_cd = np.array([cd0, cd2, cd4])
            p1 = np.polyfit(angle,coeffs_cd,2)

            CD=(p1[0] * (pitch**2+ yaw**2)) + (p1[1]*np.sqrt(pitch**2 + yaw**2)) + p1[2]

            cn0=0
            f_cn2 = interp1d(m,cn[1,:])
            cn2 = f_cd0(mach)
            f_cn4 = interp1d(m,cn[2,:])
            cn4 = f_cd2(mach)
            coeffs_cn = np.array([cn0, cn2, cn4])
            p2 = np.polyfit(angle,coeffs_cn,2)

            CNP=(p2[0]*pitch**2) +( p2[1]*pitch) + p1[2]
            CNY=(p2[0]*yaw**2) + (p2[1]*yaw) + p1[2]

            f_cp = interp1d(m,cp)
            CP = f_cp(mach)
    
    f_CN_alpha = interp1d(m,cn_alpha)
    CN_alpha = f_CN_alpha(mach)
    CP = CP/39.37 
    
    return CP, CNP, CNY, CN_alpha, CD
    
    


class Rk4Utils():
    udot = 0
    vdot = 0
    wdot = 0
    rdot = 0
    qdot = 0
    pdot = 0
    phidot = 0
    thetadot = 0
    psidot = 0
    xedot = 0
    yedot = 0
    zedot = 0

    def __init__(self, args):
        self.udot = args[0]
        self.vdot = args[1]
        self.wdot = args[2]
        self.rdot = args[3]
        self.qdot = args[4]
        self.pdot = args[5]
        self.phidot = args[6]
        self.thetadot = args[7]
        self.psidot = args[8]
        self.xedot = args[9]
        self.yedot = args[10]
        self.zedot = args[11]


from setuptools import setup

setup(
    name='helios-sim',
    version='0.1.0',
    description="6 DOF Rocket Simulator",
    author="Ronan",
    author_email="ronanmarkdsouza@gmail.com",
    packages=['helios'],
    install_requires=["numpy", "scipy", "python-dotenv", "matplotlib", "openpyxl", "pandas", "fpdf"],
    entry_points={
        "console_scripts": [
            "helios = helios.main:main"
        ]
    }

)


"""
File: ./Rk4Derivative.py

Description: This function is used to calculate derivatives of velocity components
and respective pitch, roll and yaw rates of the rocket. 

Function input:
             - thrust --> thrust of the rocket at that interval
             - drag -->  drag force experienced by the rocket at that time interval
             - sidey --> side forces along Y-axis
             - sidez --> side forces along Z-axis
             - mass --> mass of the rocket
             - u --> X-component of the velocity vector
             - v --> Y-component of the velocity vector
             - w --> Z-component of the velocity vector
             - p --> Yaw rate of the rocket in rad/s
             - q --> Pitch rate of the rocket in rad/s
             - r --> Roll rate of the rocket in rad/s
             - cphi --> cosine of the yaw angle
             - cpsi --> cosine of the roll angle
             - ctheta --> cosine of the pitch angle
             - sphi --> sine of the yaw angle
             - spsi --> sine of the roll angle
             - stheta --> sine of the pitch angle
             - ymoment --> yaw moment
             - pmoment --> pitch moment
             - rmoment --> roll moment
             - Ixx --> moment of inertia with respect to X-axis
             - Iyy --> moment of inertia with respect to Y-axis
             - Izz --> moment of inertia with respect to Z-axis

Function Output:
             - udot --> derivative of the velocity component along X-axis
             - vdot --> derivative of the velocity component along Y-axis
             - wdot --> derivative of the velocity component along Z-axis
             - rdot --> derivative of roll
             - qdot --> derivative of pitch
             - pdot --> derivative of yaw
             - phidot --> yaw rate
             - thetadot --> pitch rate
             - psidot --> roll rate
             - xedot --> derivative of displacement along X-axis
             - yedot --> derivative of displacement along Y-axis
             - zedot --> derivative of displacement along Z-axis
"""
import numpy as np

def Rk4Derivative(thrust, drag, sidey, sidez, mass, u, v, w, p, q, r, cphi, cpsi, ctheta, sphi, spsi, stheta, ymoment, pmoment, rmoment, Ixx, Iyy, Izz):
    # Eqns from PDF Titled EOM Derived
    udot = (thrust/mass) - (drag/mass) - ctheta*cpsi*9.81 + p*v - q*w
    vdot = (sidey/mass) - 9.81*(sphi*stheta*cpsi - cphi*spsi) - p*u + r*w
    wdot = (sidez/mass) - 9.81*(cphi*stheta*cpsi + sphi*spsi) + q*u - r*v
    rdot = (1/Ixx)*(rmoment + q*p*(Iyy-Izz))
    qdot = (1/Iyy)*(pmoment + r*p*(Izz-Izz))
    pdot = (1/Izz)*(ymoment + r*q*(Ixx-Iyy))
    phidot = r + (q*spsi + p*cphi)*(stheta/ctheta) 
    thetadot = q*cphi - p*sphi
    psidot = (q*sphi + p*cphi)*(1/ctheta)
    xedot = ctheta*cphi*u + (-cphi*spsi + spsi*stheta*cpsi)*v + (sphi*spsi + cphi*stheta*cpsi)*w
    yedot = ctheta*spsi*u + (cphi*cpsi + sphi*stheta*spsi)*v + (-sphi*cpsi + cphi*stheta*spsi)*w 
    zedot = -stheta*u + sphi*ctheta*v + cphi*ctheta*w
    return [udot, vdot, wdot, rdot, qdot, pdot, phidot, thetadot, psidot, xedot, yedot, zedot]